require 'spec_helper'

describe TimeInputs::Parser do
  let(:today) { Date.today }

  describe '.new' do
    it 'takes an proc as error handling policy' do
      expect { TimeInputs::Parser.new ->(error){}}.to_not raise_error
    end

    it 'uses an error raising strategy by default' do
      expect { subject }.to_not raise_error
    end
  end

  describe '#parse' do
    it 'takes one string argument and an :date_context' do
      expect { subject.parse "08:00", date_context: Date.new(2014,1,31) }
        .to_not raise_error
    end

    it 'does not require a :date_context' do
      expect { subject.parse "08:00" }.to_not raise_error
    end

    context 'given valid inputs' do
      context 'given a time in  the hh:mm format' do
        it 'converts "08:00" to a local time object on the current day' do
          expect(subject.parse "08:00")
            .to eq(Time.local(today.year, today.month, today.day,8,0))
        end

        it 'converts "14:15" to a local time object on the current day' do
          expect(subject.parse "14:15")
            .to eq(Time.local(today.year, today.month, today.day,14,15))
        end

        it 'converts "13:07", date_context: Date.new(2014,1,31) to a local time object on the given day' do
          expect(subject.parse "13:07", date_context: Date.new(2014,1,31))
            .to eq(Time.local(2014,1,31,13,7))
        end

        it 'converts "3:12", date_context: Date.new(2014,1,31) to a local time object on the given day' do
          expect(subject.parse "3:12", date_context: Date.new(2014,1,31))
            .to eq(Time.local(2014,1,31,3,12))
        end
      end

      context 'given a time in military format (e.g. 0200)' do
        it 'converts "0800" to a local time object on the current day' do
          expect(subject.parse "0800")
            .to eq(Time.local(today.year, today.month, today.day, 8, 0))
        end

        it 'converts "1415" to a local time object on the current day' do
          expect(subject.parse "1415")
            .to eq(Time.local(today.year, today.month, today.day, 14, 15))
        end

        it 'converts "1307", date_context: Date.new(2014,1,31) to a local time object on the given day' do
          expect(subject.parse "1307", date_context: Date.new(2014,1,31))
            .to eq(Time.local(2014, 1, 31, 13, 7))
        end

        it 'converts "0312", date_context: Date.new(2014,1,30) to a local time object on the given day' do
          expect(subject.parse "0312", date_context: Date.new(2014,1,30))
            .to eq(Time.local(2014, 1, 30, 3, 12))
        end
      end

      context 'given a time in the military format without leading zeros' do
        it 'converts "215" to a local time object on the current day' do
          expect(subject.parse "215")
            .to eq(Time.local(today.year, today.month, today.day, 2, 15))
        end

        it 'converts "343", date_context: Date.new(2014,1,31) to a local time object on the given day'do
          expect(subject.parse "343", date_context: Date.new(2014,1,31))
            .to eq(Time.local(2014,1,31,3,43))
        end
      end
    end

    context 'given invalid inputs' do
      let(:error_policy) { double('error_handler') }
      subject { TimeInputs::Parser.new error_policy }

      context 'given invalid parse inputs' do
        it 'calls the error handler with "invalid input"' do
          expect(error_policy).to receive(:call).with("invalid input")
          subject.parse "invalid input"
        end
      end

      context 'given an invalid date_context: object' do
        it 'calls the error handler with "invalid date_context"' do
          expect(error_policy).to receive(:call).with("invalid date_context")
          subject.parse "1605", date_context: Object.new
        end
      end
    end
  end
end
